﻿package com.wastedpotential {
	import flash.display.MovieClip;
	import flash.display.Graphics;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.events.Event;
	import flash.events.MouseEvent;	
	
	public class MaskTestMain extends MovieClip {
		
		private var enableMasking:Boolean = true; //set this to false to turn off masking and just draw lines
		private var tempGraphics:Graphics;
		
		//- CONSTRUCTOR - sets up dynamic drawing (and masking)
		public function MaskTestMain() {
			//reset masking:
			updateMask();
			
			//set up drawing:
			tempGraphics = drawingBoard_mc.graphics; //drawClip_mc.graphics;
			tempGraphics.clear();
			tempGraphics.lineStyle(30, 0xff0000, 1); //set up line style for drawing
			
			//add drawing listeners:
			stage.addEventListener(MouseEvent.MOUSE_DOWN, onStartDrawing);
			stage.addEventListener(MouseEvent.MOUSE_UP, onStopDrawing);
		}
		
		//- start drawing lines:
		private function onStartDrawing(e:MouseEvent):void {
			tempGraphics.moveTo(mouseX, mouseY);
			addEventListener(Event.ENTER_FRAME, onDrawFrame);
		}
		
		//- stop drawing lines:
		private function onStopDrawing(e:MouseEvent):void {
			removeEventListener(Event.ENTER_FRAME, onDrawFrame);
		}					
					
		//- called on every frame while drawing - draws the next line and calls masking function:
		private function onDrawFrame(e:Event):void {
			tempGraphics.lineTo(mouseX, mouseY);	
			updateMask(); 
		}		
		
		//- when masking is enabled, this uses the dynamic lines to create a mask:
		private function updateMask():void {
			if (enableMasking) {
				//remove all old masks from the maskHolder:
				while (maskHolder_mc.numChildren > 0) {
					maskHolder_mc.removeChildAt(0);
				}
				
				objectToMask_mc.mask = null; //clear the previous mask form the object
				
				//parameters of bitmapData MUST be set to: ([width], [height], true, 0) to get the transparency correct:
				var myBitmapData:BitmapData = new BitmapData(stage.stageWidth, stage.stageHeight, true, 0);
				
				drawingBoard_mc.visible = true;	//make this visible for creating the bitmap	
				myBitmapData.draw(drawingBoard_mc); //create a bitmap copy of the drawing board
				drawingBoard_mc.visible = false; //hide the drawing board again
				var printBitmap:Bitmap = new Bitmap(myBitmapData); //create a bitmap from the bitmap data
				printBitmap.smoothing = true; //optional
				var maskClip:MovieClip = new MovieClip; //create a movieclip to act as the new mask
				maskClip.addChild(printBitmap); //add the bitmap copy of the drawing board to the new mask movieclip
				maskClip.cacheAsBitmap = true; //IMPORTANT: mask movieclip cacheAsBitmap = true or this doesn't work
				maskHolder_mc.addChild(maskClip); //add the new mask to the maskHolder
				objectToMask_mc.cacheAsBitmap = true; //IMPORTANT: masked movieclip cacheAsBitmap = true or this doesn't work
				objectToMask_mc.mask = maskClip; //set our new mask on the object 
			}
		}			
		
	} //end class
} //end package